from .document import Document
from .audio import Audio
from .contact import ContactMessage
from .location import Location
from .photo import Photo

__all__ = (
	"Document",
	"Audio",
	"ContactMessage",
	"Location",
	"Photo"
)
